/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package tracking.trackerEnsemble.impl;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import static java.util.Arrays.fill;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import tracking.trackerEnsemble.exception.*;

/**
 *
 * @author winsty
 */
public class FactorialHMMEnsembler extends AbstractTrackerEnsembler {

    private final String OUTPUT_PATH_NAME;
    private final String VIDEO_STATS_NAME;
    private final double NORMAL_HEIGHT;
    private final double NORMAL_WIDTH;
    private final double scaleWidth;
    private final double scaleHeight;
    private final int numY;
    private final int numZ;
    private final double a;
    private final double b;
    private final double alpha;
    private final double[][][] confTracker;
    private final double[][][] particleTracker;
    private final double[][] param;
    private final double[] conf;
    private final double[] affsig;
    private final int[] failedCounter;
    private final int[] failedSince;
    private final double confThres;
    private final int pruneNum;
    private final int lowReliabilityPeriod;
    private BufferedWriter writer;
    private int currentFrame;
    private BoundingBox initBB;
    private final Randoms rand;
    private double w;
    private double h;
    
    public FactorialHMMEnsembler(int trackerNum, String videoName) throws ConfigNotFoundException{
        super(trackerNum);
        
        Properties prop = new Properties();
		InputStream input = null;
 
	// Parse the parameters, we don't handle the exceptions for invalid args here.
	try {
            input = this.getClass().getClassLoader().getResourceAsStream("FHMMEnsembler.properties"); 
            prop.load(input);
            OUTPUT_PATH_NAME = prop.getProperty("output_path_name");
            VIDEO_STATS_NAME = prop.getProperty("video_stats_name");
            NORMAL_HEIGHT = Double.parseDouble(prop.getProperty("normal_height"));
            NORMAL_WIDTH = Double.parseDouble(prop.getProperty("normal_width"));
            numY = Integer.parseInt(prop.getProperty("Y"));
            numZ = Integer.parseInt(prop.getProperty("Z"));
            a = Double.parseDouble(prop.getProperty("a"));
            b = Double.parseDouble(prop.getProperty("b"));
            alpha = Double.parseDouble(prop.getProperty("alpha"));
            confThres = Double.parseDouble(prop.getProperty("confThres"));
            pruneNum = Integer.parseInt(prop.getProperty("pruneNum"));
            lowReliabilityPeriod = Integer.parseInt(prop.getProperty("lowReliabilityPeriod"));
            String[] temp = prop.get("affsig").toString().split(",");
            affsig = new double[4];
            for (int i = 0; i < 4; ++i){
                affsig[i] = Double.parseDouble(temp[i]);
            }
	} catch (IOException ex) {
		throw new ConfigNotFoundException("Cannot find FHMMEnsembler.properties");
	} finally {
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    // Do Nothing
                }
            }
	}

        confTracker = new double[trackerNum][numY][numZ];
        particleTracker = new double[trackerNum][numY][numZ];
        for (int i = 0; i < trackerNum; ++i){
            for (int j = 0; j < numY; ++j){
                for (int k = 0; k < numZ; ++k){
                    confTracker[i][j][k] = 1.0 / numZ;
                    particleTracker[i][j][k] = 1.0;
                }
            }
        }
        
        // Note that the benchmark will add a suffix automatically, we need to 
        // remove it.
        videoName = videoName.split("_")[0];
        File statsFile = new File(VIDEO_STATS_NAME + videoName + ".txt");
        BufferedReader reader;

        try{
            reader = new BufferedReader(new FileReader(statsFile));
            String temp = reader.readLine();
            String[] s = temp.split(" ");
            double height = Integer.parseInt(s[0]);
            double width = Integer.parseInt(s[1]);
            scaleWidth = width / NORMAL_WIDTH;
            scaleHeight = height / NORMAL_HEIGHT;
        } catch (FileNotFoundException ex) {
            throw new ConfigNotFoundException("Cannot find " + statsFile);
        } catch (IOException ex) {
            throw new ConfigNotFoundException("Cannot find " + statsFile);
        }


        currentFrame = 0;
        
        conf = new double[numY];
        param = new double[numY][4];
        
        failedCounter = new int[trackerNum];
        failedSince = new int[trackerNum];
        for (int i = 0; i < trackerNum; ++i){
            failedSince[i] = lowReliabilityPeriod;
        }
        
        rand = new Randoms();
        
        try {
            File f = new File(OUTPUT_PATH_NAME + videoName + ".txt");
            f.createNewFile();
            writer = new BufferedWriter(new FileWriter(f));
        } catch (IOException ex) {
            writer = null;
            java.util.logging.Logger.getLogger(FactorialHMMEnsembler.class.getName()).log(Level.WARNING, null, ex);
        }
    }

    
    @Override
    public void ensemble() {
        int[] idx;
        if (currentFrame == 0){
            double tempX = 0, tempY = 0, tempW = 0, tempH = 0;
            for (int i = 0; i < trackerNum; ++i){
                tempX += bbs.get(i).getCenterX();
                tempY += bbs.get(i).getCenterY();
                tempW += bbs.get(i).getWidth();
                tempH += bbs.get(i).getHeight();
            }
            initBB = new BoundingBox(tempX / trackerNum, tempY / trackerNum,
                    tempW / trackerNum, tempH / trackerNum);
            w = initBB.getWidth() / scaleWidth;
            h = initBB.getHeight() / scaleHeight;
            
            idx = new int[numY];
            fill(idx, 0);
            param[0] = Helper.corner2geom(initBB, scaleWidth, scaleHeight);
        } else {
            idx = Helper.resample(conf, numY);
        }
        double[][] oldParam = Helper.clone2dim(param);
        double[][][] oldConfTracker = Helper.clone3dim(confTracker);
        double[][][] oldParticleTracker = Helper.clone3dim(particleTracker);
        for (int i = 0; i < numY; ++i){
            param[i][0] = oldParam[idx[i]][0] + rand.nextGaussian()* affsig[0];
            param[i][1] = oldParam[idx[i]][1] + rand.nextGaussian() * affsig[1];
            param[i][2] = oldParam[idx[i]][2] + rand.nextGaussian() * affsig[2];
            param[i][3] = oldParam[idx[i]][3] + rand.nextGaussian() * affsig[3];
            double[] CPEs = new double[trackerNum];
            double[] ORs = new double[trackerNum];
            double cumConf = 1;
            for (int j = 0; j < trackerNum; ++j){
                CPEs[j] = Helper.calcCPE(param[i], Helper.corner2geom(bbs.get(j), scaleWidth, scaleHeight), w, h)
                        * alpha;
                ORs[j] = Helper.calcOR(param[i], Helper.corner2geom(bbs.get(j), scaleWidth, scaleHeight));
                
                int[] idxZ = Helper.resample(oldConfTracker[j][idx[i]], numZ);
                double sum = 0;
                for (int k = 0; k < numZ; ++k){
                    
                    double particle = oldParticleTracker[j][idx[i]][idxZ[k]];
                    double temp = rand.nextGamma(particle / b, b);
                    if (temp < 1e-10) temp = 1e-10;
                    particleTracker[j][i][k] = temp;
                    confTracker[j][i][k] = Math.exp(-0.5 * CPEs[j] * CPEs[j] * temp - temp * (1 - ORs[j]) - temp * a)
                            * temp * temp / (1 - Math.exp(-temp)) ;
                    sum += confTracker[j][i][k];
                } 
                cumConf *= sum;
                for (int k = 0; k < numZ; ++k){
                    confTracker[j][i][k] /= sum;
                }
            }
            conf[i] = cumConf;
        }
        double totalConf = 0;
        for (int i = 0; i < numY; ++i){
            totalConf += conf[i];
        }
        if (totalConf == 0){
            totalConf = 1e-10;
        }
        for (int i = 0; i < numY; ++i){
            conf[i] /= totalConf;
        }
        double bestConf = -1;
        int bestConfId = 0;
        for (int i = 0; i < numY; ++i){
            if (conf[i] > bestConf){
                bestConf = conf[i];
                bestConfId = i;
            }
        }
        res = Helper.geom2corner(param[bestConfId], scaleWidth, scaleHeight);
        w = res.getWidth() / scaleWidth;
        h = res.getHeight() / scaleHeight;
        
        //Deal with failure case
        if (res.getCenterX()< 0){
            res.setCenterX(0);
        }
        if (res.getCenterX() < 0){
            res.setCenterY(0);
        }
        if (res.getCenterY() > NORMAL_HEIGHT * scaleHeight){
            res.setCenterY(NORMAL_HEIGHT * scaleHeight);
        }
        if (res.getCenterX() > NORMAL_WIDTH * scaleWidth){
            res.setCenterX(NORMAL_WIDTH * scaleWidth);
        }
        
        if (writer != null){
            try {
                writer.write(res.toString());
                writer.write("\n");
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(FactorialHMMEnsembler.class.getName()).log(Level.WARNING, null, ex);
            }
        }
        
        // Check whether the tracker has already failed.
        for (int i = 0; i < trackerNum; ++i){
            double trackerConf = 0;
            for (int j = 0; j < numY; ++j){
                for (int k = 0; k < numZ; ++k){
                    trackerConf += confTracker[i][j][k] * particleTracker[i][j][k];
                }
            }
            System.out.println(trackerConf / numY);
            System.out.println(Math.pow(confThres, trackerNum));
            if (trackerConf / numY < Math.pow(confThres, trackerNum)){
                ++failedCounter[i];
            } else {
                failedCounter[i] = 0;    
            }
            ++failedSince[i];
        }
        ++currentFrame;
    }

    @Override
    public BoundingBox getResult(int trackerId) {
        if (failedCounter[trackerId] >= pruneNum){
            failedCounter[trackerId] = 0;
            failedSince[trackerId] = 0;
        }
        if (failedSince[trackerId] < lowReliabilityPeriod){
            return res;
        } else {
            return bbs.get(trackerId);
        }
    }

    @Override
    public void endTracking() {
        if (writer != null){
            try {
                writer.close();
                writer = null;
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(FactorialHMMEnsembler.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    
    
}
