/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package tracking.trackerEnsemble.impl;

import static java.util.Arrays.sort;
import java.util.Random;

/**
 *
 * @author winsty
 */
public class Helper<T> {
     public static final int TMPL_WIDTH = 32;
     public static final int TMPL_HEIGHT = 32;
    
    public static int[] resample(double[] conf, int n){
        double[] cumsum = new double[n];
        cumsum[0] = conf[0];
        for (int i = 1; i < n; ++i){
            cumsum[i] = cumsum[i - 1] + conf[i];
        }
        cumsum[n - 1] = 1;
        Random rand = new Random();
        double[] r = new double[n];
        for (int i = 0; i < n; ++i){
            r[i] = rand.nextDouble();
        }
        sort(r);
        int pos = 0;
        int[] out = new int[n];
        for (int i = 0; i < n; ++i){
            while (pos < n && r[pos] <= cumsum[i]){
                out[pos++] = i;
            }
        }
        return out;
    }
    
    public static double[] corner2geom(BoundingBox bb, double scaleWidth, double scaleHeight){
        double[] ret = new double[4];
        ret[0] = bb.getCenterX() / scaleWidth;
        ret[1] = bb.getCenterY() / scaleHeight;
        ret[2] = bb.getWidth() / Helper.TMPL_WIDTH / scaleWidth;
        ret[3] = bb.getHeight() / bb.getWidth() / (Helper.TMPL_HEIGHT / Helper.TMPL_WIDTH) / scaleHeight * scaleWidth;
        return ret;
    }
    
    public static BoundingBox geom2corner(double[] geom, double scaleWidth, double scaleHeight){
        double[] ret = new double[4];
        ret[0] = geom[0] * scaleWidth;
        ret[1] = geom[1] * scaleHeight;
        ret[2] = geom[2] * Helper.TMPL_WIDTH * scaleWidth;
        ret[3] = ret[2] * geom[3] * (Helper.TMPL_HEIGHT / Helper.TMPL_WIDTH) * scaleHeight / scaleWidth;
        return new BoundingBox(ret[0], ret[1], ret[2], ret[3]);
    }
    
    public static double calcCPE(double[] bb1, double[] bb2, double w, double h){
        return Math.sqrt((bb1[0] - bb2[0]) *  (bb1[0] - bb2[0]) / w / w
                + (bb1[1] - bb2[1]) * (bb1[1] - bb2[1]) / h / h);
    }
    
    public static double calcOR(double[] goem1, double[] goem2){
        BoundingBox bb1 = geom2corner(goem1, 1, 1);
        BoundingBox bb2 = geom2corner(goem2, 1, 1);
        double minX = Math.max(bb1.getLeftTopX(), bb2.getLeftTopX());
        double maxX = Math.min(bb1.getRightBottomX(), bb2.getRightBottomX());
        double minY = Math.max(bb1.getLeftTopY(), bb2.getLeftTopY());
        double maxY = Math.min(bb1.getRightBottomY(), bb2.getRightBottomY());
        double overlapping = Math.max(0, maxX - minX) * Math.max(0, maxY - minY);
        return overlapping / (bb1.getWidth() * bb1.getHeight() + bb2.getWidth() * bb2.getHeight() - overlapping);
    }
    
    public static double[][] clone2dim(double[][] src){
        double[][] dst = src.clone();
        for (int i = 0; i < dst.length; ++i){
            dst[i] = src[i].clone();
        }
        return dst;
    }
    
    public static double[][][] clone3dim(double[][][] src){
        double [][][] dst = src.clone();
        for (int i = 0; i < dst.length; ++i){
            dst[i] = src[i].clone();
            for (int j = 0; j < src[i].length; ++j){
                dst[i][j] = src[i][j].clone();
            }
        }
        return dst;
    }
}
