/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package tracking.trackerEnsemble.webservice;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import javax.jws.WebService;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.soap.SOAPBinding;
import javax.jws.soap.SOAPBinding.Style;
import tracking.trackerEnsemble.impl.*;
import tracking.trackerEnsemble.exception.*;
/**
 *
 * @author winsty
 */
@WebService(serviceName = "trackerEnsemble")
@SOAPBinding(style = Style.RPC)
public class TrackerEnsembleService {

    private final HashMap<String, Integer> trackerMapping = new HashMap<String, Integer>();
    
    private final Lock updateLock = new ReentrantLock();
    private final Lock finishLock = new ReentrantLock();
    private final Lock registerLock = new ReentrantLock();
    private final ArrayList<Integer> counter = new ArrayList<Integer>();
    private final ArrayList<Boolean> registered = new ArrayList<Boolean>();
    private final ArrayList<Integer> unregistered = new ArrayList<Integer>();

    private AbstractTrackerEnsembler ensembler;
    private final int trackerNum;
    private int currentFrame;
    private int totalVideoNum;
    private String videoName;
    
    public TrackerEnsembleService() throws ConfigNotFoundException{
        Properties prop = new Properties();
	InputStream input = null;
 
	try {
            String path = getClass().getClassLoader().getResource("/").getPath();  
            System.out.print(path);
            input = this.getClass().getClassLoader().getResourceAsStream("service.properties"); 
            prop.load(input);

            // get the property value and print it out
            System.out.println(prop.getProperty("trackers"));
            String[] trackers = prop.get("trackers").toString().split(",");
            trackerNum = trackers.length;
            for (int i = 0; i < trackerNum; ++i){
                trackerMapping.put(trackers[i], i);
            }
	} catch (IOException ex) {
		throw new ConfigNotFoundException("Cannot find service.properties");
	} finally {
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    // Do Nothing
                }
            }
	}
        for (int i = 0; i < trackerNum; ++i){
            counter.add(0);
            registered.add(false);
            unregistered.add(0);
        }
        totalVideoNum = 0;
        videoName = "";
    }
    /**
     * Web service operation
     */
    @WebMethod(operationName = "registerTracker")
    public Integer registerTracker(@WebParam(name = "trackerName") String trackerName, 
            @WebParam(name = "videoName") String videoName) throws TrackerNotFoundException, InterruptedException, VideoNameNotMatchException, IOException, ConfigNotFoundException {
        
        if (!trackerMapping.containsKey(trackerName)){
            throw new TrackerNotFoundException();
        }
        
        if (!"".equals(this.videoName) && !this.videoName.equals(videoName)){
            throw new VideoNameNotMatchException();
        }
        
        try {
            registerLock.lock();
            if ("".equals(this.videoName)){
                ensembler = new FactorialHMMEnsembler(trackerNum, videoName);
                this.videoName = videoName;
            }
            System.out.println(trackerName + " " + this.videoName);
            int trackerId = trackerMapping.get(trackerName); 
            registered.set(trackerId, true);
        }
        finally {
            registerLock.unlock();
        }
        while (true){    
            if (checkAllRegistered()){
                break;
            }
            Thread.sleep(50);
        }
        return trackerMapping.get(trackerName);
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "update")
    public String update(@WebParam(name = "trackerId") int trackerId, @WebParam(name = "centerX") double centerX, @WebParam(name = "centerY") double centerY, 
            @WebParam(name = "width") double width, @WebParam(name = "height") double height) throws InterruptedException, TrackerNotRegisteredException {
        if (trackerId < 0 || trackerId >= trackerNum || !registered.get(trackerId)){
            throw new TrackerNotRegisteredException();
        }
        Boolean ready = false;
        synchronized (counter){
            counter.set(trackerId, counter.get(trackerId) + 1);
        }
        synchronized (ensembler){
            ensembler.setResult(trackerId, new BoundingBox(centerX, centerY, width, height));
        }
        while (!ready){
            updateLock.lock();
            try{
                ready = checkReady(trackerId);
            } finally{
                updateLock.unlock();
            }
            Thread.sleep(50);
        }
        return ensembler.getResult(trackerId).toString(); 
    }
    
    /**
     * Web service operation
     */
    @WebMethod(operationName = "finishTracking")
    public Boolean finishTracking(@WebParam(name = "trackerId") int trackerId) throws InterruptedException {
        synchronized (registered){
            unregistered.set(trackerId, unregistered.get(trackerId) + 1);
        }
        Boolean allFinished = false;
        while (!allFinished){
            try{
                finishLock.lock();
                allFinished = checkFinished(trackerId);
                Thread.sleep(50);
            } finally {
                finishLock.unlock();
            }
        }
        
        return true;
    }
    
    private synchronized Boolean checkReady(int id){
        synchronized (counter){
            for (int i = 0; i < counter.size(); ++i){
                if (counter.get(i) < counter.get(id)){
                    return false;
                }
            }
            if (currentFrame < counter.get(id)){
                System.out.println(id);
                System.out.println(currentFrame);
                ensembler.ensemble();
                ++currentFrame;
            }
            return true;
        }
    }

    private boolean checkAllRegistered() {
        for (int i = 0; i < registered.size(); ++i){
            if (!registered.get(i)){
                return false;
            }
        }
        return true;
    }

    private synchronized boolean checkFinished(int id) {
        for (int i = 0; i < registered.size(); ++i){
            if (unregistered.get(i) < unregistered.get(id)){
                return false;
            }
        }
        counter.set(id, 0);
        if (unregistered.get(id) > totalVideoNum){
            ensembler.endTracking();
            videoName = "";
            currentFrame = 0;
            ++totalVideoNum;
        }
        return true;
    }
}
