function results = run_EBT(seq, res_path, bSaveImage)
    load(['data/' seq.name '.mat']);
    init;
    confTracker = ones(opts.numTrackers, opts.numY, opts.numZ) / opts.numZ;
    particleTracker = ones(opts.numTrackers, opts.numY, opts.numZ) * 1;
    res = cell(seq.len, 1);
    pruned = zeros(opts.numTrackers, 1);
    counter = zeros(opts.numTrackers, 1);
    
    % Init the bb in first frame
    rect=seq.init_rect;
    p = [rect(1)+rect(3)/2, rect(2)+rect(4)/2, rect(3), rect(4), 0]; 
    paramOld = [p(1), p(2), p(3)/opts.tmplsize(2), p(5), p(4) /p(3) / (opts.tmplsize(1) / opts.tmplsize(2)), 0];
    param.est = affparam2mat(paramOld);
    frame = imread(seq.s_frames{1});             %%Load the first frame
    scaleHeight = size(frame, 1) / opt.normalHeight;
    scaleWidth = size(frame, 2) / opt.normalWidth;  
    result = [];
    for f = 1 : seq.len
        im = imread(seq.s_frames{f});
        tic;
        if ~isfield(param,'param')
            param.param(1, :) = affparam2geom(param.est(:));
            idx = ones(opts.numY, 1);
        else
            idx = resample(param.conf, opts.numY);
        end
        oldParam = param;
        oldConfTracker = confTracker;
        oldparticleTracker = particleTracker;
        for i = 1 : opts.numY
            bb = oldParam.param(idx(i), :) + randn(1, 6) .* opts.affsig(:)'...
                .* [scaleWidth, scaleHeight, scaleWidth, 1, scaleHeight / scaleWidth, 1];
            param.param(i, :) = bb;
            temp = affine2corner(bb, opts);
            CPEs = calcCPE(bb, Z{f}, temp(1, 2) - temp(1, 1), temp(2, 3) - temp(2, 2)) * opts.alpha;
            ORs = calcOR(bb, Z{f}, opts);	
            unnormalizedConf = 1;
            OPTS_KS = opts.k * ones(opts.numZ, 1);
            for j = 1 : opts.numTrackers
                if pruned(j)
                    continue;
                end
                particles = oldparticleTracker(j, idx(i), resample(oldConfTracker(j, idx(i), :), opts.numZ));
                particles = particles(:);
                temp = opts.k .* randg(particles / opts.k, [opts.numZ, 1]); 
                temp(temp < 1e-10) = 1e-10;
                particleTracker(j, i, :) = temp;
                confs = exp(-0.5 * CPEs(j) * CPEs(j) .* temp - temp .* (1 - ORs(j)) - temp * opts.a)...
                    .* temp .* temp ./ (1 - exp(-temp));
                confs(isnan(confs)) = 0;
                temp = sum(confs) + 1e-10;
                unnormalizedConf = unnormalizedConf * temp;
                confTracker(j, i, :) = confs / temp;
            end
            param.conf(i) = unnormalizedConf;
        end
        param.conf = param.conf / sum(param.conf);
        [~, id] = max(param.conf);
        param.est = affparam2mat(param.param(id,:));
        result = [result; param.est];
        res{f} = affine2corner(param.param(id,:), opts);
        temp = sum(sum(confTracker .* particleTracker, 2), 3) / opts.numY;
        disp(temp);
    %     disp(pruned');
    %     disp('--------------');
        counter = counter + (temp <= opts.confThres ^ opts.numTrackers);
        counter(temp > opts.confThres ^ opts.numTrackers) = 0;
        pruned(counter >= opts.pruneNum) = true;
        elstime = toc;
        fprintf('frame %d, %.2ffps\n', f, 1 / elstime);

        imshow(im);
        rectangle('Position',[res{f}(1, 1) res{f}(2, 1) (res{f}(1, 2) - res{f}(1, 1)) (res{f}(2, 4) - res{f}(2, 1))],...
            'LineWidth',2,'EdgeColor','r');
        hold on;
        text(5, 18, strcat('#',num2str(f)), 'Color','y', 'FontWeight','bold', 'FontSize',20);
        set(gca,'position',[0 0 1 1]); 
        pause(0.0001);
        hold off;
        drawnow;  
    end
    results.res=result;
    results.type='ivtAff';
    results.tmplsize = [32 32];
    results.fps = 5;
end
