function [distance coeff err] = lss_distance(data, A, PA, param)
%%  function [distance coeff err] = lss_distance(data, A, PA, param)
%%      Calculate the Least Soft-thresold Squares Regression
%%  Input:  
%%          data:       An n-by-1 Column Vector
%%          A:          An n-by-d Matrix
%%          PA:         An d-by-n Matrix    PA = (inv(A'*A))*A'
%%          param:      Parameters for Sloving Soft-thresold Squares Regression
%%              -param.lambda:      For Penalizing the Laplacian Noise Term
%%              -param.maxLoopNum   A Maximal Number of Iterations
%%              -param.tol          Stopping Criterion
%%  Output:
%%          distance:   The Least Soft-thresold Squares Distance
%%          coeff:      Regression Coefficient
%%          err:        Laplacian Noise Term
%%
%%DUT-IIAU-DongWang-2013-03-27
%%Dong Wang, Huchuan Lu, Minghsuan Yang, Least Soft-thresold Squares
%%Tracking. CVPR 2013.
%%http://ice.dlut.edu.cn/lu/index.html
%%wangdong.ice@gmail.com
%%

%%1.Initialization:
coeff  = zeros(size(A,2),1);
err    = zeros(size(data));
errT   = zeros(size(data));
disOld = 0;
disNew = 0;

%%2.Iterative solution: 
for num = 1:param.maxLoopNum
    %(2.1)Fix 'err', slove 'coeff':
    coeff  = PA*(data-err);
    %(2.2)Fix 'coeff', slove 'err':
    errT   = data - A*coeff;
    err    = max(abs(errT)-param.lambda, 0).*sign(errT);
    %(2.3)Judge convergence:
    disNew = 0.5*sum((errT-err).^2)+param.lambda*sum(abs(err));
    if  abs(disNew-disOld) < param.tol,
        break;
    end
    disOld = disNew;
end
distance = disNew;