%%DUT-IIAU-DongWang-2013-03-27
%%Dong Wang, Huchuan Lu, Minghsuan Yang, Least Soft-thresold Squares
%%Tracking. CVPR 2013.
%%http://ice.dlut.edu.cn/lu/index.html
%%wangdong.ice@gmail.com
%%
% clear all;
% clc;
function results=run_LSSTEnsemble(seq, res_path, bSaveImage)

close all;
opt.normalHeight = 240;
opt.normalWidth = 320;
opt.tmplsize = [32, 32];
opt.numsample = 600;
opt.affsig = [4,4,0.01,0.0,0.001,0];
opt.lssParam = [];                           
opt.lssParam.lambda = 0.1;          %%For Penalizing the Laplacian Noise Term
opt.lssParam.maxLoopNum = 20;       %%A Maximal Number of Iterations
opt.lssParam.tol        = 0.001;    %%Stopping Threshold     
opt.condenssig = 0.05;
opt.maxbasis = 16;
opt.ff = 1.0;
opt.batchsize = 5; 
%%***********************1.Initialization****************************%%

%1.2 Load functions and parameters:
rect=seq.init_rect;
p = [rect(1)+rect(3)/2, rect(2)+rect(4)/2, rect(3), rect(4), 0]; 
TotalFrameNum = seq.len;                        %%Total frame number
frame = imread(seq.s_frames{1});             %%Load the first frame
scaleHeight = size(frame, 1) / opt.normalHeight;
scaleWidth = size(frame, 2) / opt.normalWidth;  
frame = imresize(frame, [opt.normalHeight, opt.normalWidth]);

p(1) = p(1) / scaleWidth;
p(3) = p(3) / scaleWidth;
p(2) = p(2) / scaleHeight;
p(4) = p(4) / scaleHeight;
param0 = [p(1), p(2), p(3)/32, p(5), p(4)/p(3), 0];      
param0 = affparam2mat(param0);

if  size(frame,3) == 3
    framegray = double(rgb2gray(frame))/256;    %%For color images
else
    framegray = double(frame)/256;              %%For Gray images
    frame = cat(3, [], frame, frame, frame);
end

%1.3 Load functions and parameters:
tmpl.mean = warpimg(framegray, param0, opt.tmplsize);    
tmpl.basis = [];                                        
tmpl.eigval = [];                                       
tmpl.numsample = 0;                                    
%
param = [];
param.est = param0;                                     
param.wimg = tmpl.mean; 
param.errRatio = [];
% draw initial track window
drawopt = drawtrackresult([], 0, frame, tmpl, param.est);
% disp('resize the window as necessary, then press any key..'); pause;

wimgs = [];     %%Data buffer
result = [];    %%Tracking results
duration = 0; 
ensembleService = trackerEnsemble;
trackerId = registerTracker(ensembleService, 'LSST', seq.name);
for num = 1:TotalFrameNum
    %%2.1 Load the (num)-th frame
    frame = imread(seq.s_frames{num});
    
    if  size(frame,3) == 3
        framegray = double(rgb2gray(frame))/256;
    else
        framegray = double(frame)/256;
        frame = cat(3, [], frame, frame, frame);
    end
    framegray = imresize(framegray, [opt.normalHeight, opt.normalWidth]);
    tic;
    %%2.2 Do tracking
    param = estwarp_condens_LSS_PCA(framegray, tmpl, param, opt);
    
    oldEst = param.est;
    [p, pOri] = updateToMaster(param.est', scaleWidth, scaleHeight, ensembleService, trackerId, opt);
    paramOri = [pOri(1), pOri(2), pOri(3)/opt.tmplsize(2), pOri(5), pOri(4) /pOri(3) / (opt.tmplsize(1) / opt.tmplsize(2)), 0];
    result = [result; affparam2mat(paramOri)];

    if sum(abs(oldEst(1) - p(1)) + abs(oldEst(2) - p(2))) > 1
        disp('reset');
        paramOld = [p(1), p(2), p(3)/opt.tmplsize(2), p(5), p(4) /p(3) / (opt.tmplsize(1) / opt.tmplsize(2)), 0];
        param.est = affparam2mat(paramOld)';
        param = rmfield(param, 'param');
        param.wimg = warpimg(framegray, param.est, size(tmpl.mean));
    end
    
%     res = affparam2geom(param.est');
%     p(1) = round(res(1));
%     p(2) = round(res(2)); 
%     p(3) = round(res(3) * opt.tmplsize(2));
%     p(4) = round(res(5) * (opt.tmplsize(1) / opt.tmplsize(2)) * p(3));
%     p(5) = res(4);
%     p(1) = p(1) * scaleWidth;
%     p(3) = p(3) * scaleWidth;
%     p(2) = p(2) * scaleHeight;
%     p(4) = p(4) * scaleHeight;
%     paramOld = [p(1), p(2), p(3)/opt.tmplsize(2), p(5), p(4) /p(3) / (opt.tmplsize(1) / opt.tmplsize(2)), 0];
%     result = [result; affparam2mat(paramOld)];
%     p1(1) = round(res(1));
%     p1(2) = round(res(2)); 
%     p1(3) = round(res(3) * opt.tmplsize(2));
%     p1(4) = round(res(5) * (opt.tmplsize(1) / opt.tmplsize(2)) * p1(3));
%     p1(5) = res(4);
%     res = update(ensembleService, trackerId, p1(1), p1(2), p1(3), p1(4));
%     C = strsplit(res, ',');
%     p = [str2double(C(1)) str2double(C(2)) str2double(C(3)) str2double(C(4)) 0];
%     paramOld = [p(1), p(2), p(3)/opt.tmplsize(2), p(5), p(4) /p(3) / (opt.tmplsize(1) / opt.tmplsize(2)), 0];
%     if sum(p ~= p1) > 0
%         param.est = affparam2mat(paramOld)';
%         param = rmfield(param, 'param');
%         param.wimg = warpimg(framegray, param.est, size(tmpl.mean));
%     end
    % Rewrap image here!!!!
    
%     result = [ result; param.est' ];
    
    
    %%2.3 Update model
    wimgs = [wimgs, param.wimg(:)]; 
    if  (size(wimgs,2) >= opt.batchsize && ~isempty(wimgs))   
        %%(1)Incremental PCA
        [tmpl.basis, tmpl.eigval, tmpl.mean, tmpl.numsample] = ...
        sklm(wimgs, tmpl.basis, tmpl.eigval, tmpl.mean, tmpl.numsample, opt.ff);   
        %%(2)Clear data buffer
        wimgs = [];     
        %%(3)Keep (opt.maxbasis) basis vectors
        if  (size(tmpl.basis,2) > opt.maxbasis)          
            tmpl.basis  = tmpl.basis(:,1:opt.maxbasis);   
            tmpl.eigval = tmpl.eigval(1:opt.maxbasis);
        end
    end
    duration = duration + toc; 
    %%2.4 Draw tracking results
    drawopt = drawtrackresult(drawopt, num, frame, tmpl, result(end, :));   
end     

finishTracking(ensembleService, trackerId);
fprintf('%d frames took %.3f seconds : %.3fps\n',num, duration, num/duration);
fps = num/duration;

results.res=result;
results.type='ivtAff';
results.tmplsize = [32 32];
results.fps = fps;
disp(['fps: ' num2str(results.fps)])

end